@@ -9,15 +9,16 @@
 # Spyder consoles sitecustomize
 #
 
-import sys
+import bdb
+from distutils.version import LooseVersion
+import io
 import os
 import os.path as osp
 import pdb
-import bdb
-import time
-import traceback
 import shlex
-
+import sys
+import time
+import warnings
 
 PY2 = sys.version[0] == '2'
 
@@ -34,8 +35,7 @@ if not hasattr(sys, 'argv'):
 #==============================================================================
 # Main constants
 #==============================================================================
-IS_IPYKERNEL = os.environ.get("IPYTHON_KERNEL", "").lower() == "true"
-IS_EXT_INTERPRETER = os.environ.get('EXTERNAL_INTERPRETER', '').lower() == "true"
+IS_EXT_INTERPRETER = os.environ.get('SPY_EXTERNAL_INTERPRETER') == "True"
 
 
 #==============================================================================
@@ -102,40 +102,6 @@ except ImportError:
             exec(compile(f.read(), filename, 'exec'), namespace)
 
 
-#==============================================================================
-# Colorization of sys.stderr (standard Python interpreter)
-#==============================================================================
-if os.environ.get("COLORIZE_SYS_STDERR", "").lower() == "true":
-    class StderrProxy(object):
-        """Proxy to sys.stderr file object overriding only the `write` method
-        to provide red colorization for the whole stream, and blue-underlined
-        for traceback file links"""
-        def __init__(self):
-            self.old_stderr = sys.stderr
-            self.__buffer = ''
-            sys.stderr = self
-
-        def __getattr__(self, name):
-            return getattr(self.old_stderr, name)
-
-        def write(self, text):
-            if os.name == 'nt' and '\n' not in text:
-                self.__buffer += text
-                return
-            for text in (self.__buffer+text).splitlines(True):
-                if text.startswith('  File') \
-                and not text.startswith('  File "<'):
-                    # Show error links in blue underlined text
-                    colored_text = '  '+'\x1b[4;34m'+text[2:]+'\x1b[0m'
-                else:
-                    # Show error messages in red
-                    colored_text = '\x1b[31m'+text+'\x1b[0m'
-                self.old_stderr.write(colored_text)
-            self.__buffer = ''
-
-    stderrproxy = StderrProxy()
-
-
 #==============================================================================
 # Prepending this spyder package's path to sys.path to be sure
 # that another version of spyder won't be imported instead:
@@ -168,13 +134,14 @@ if os.name == 'nt' and PY2:
         except (ValueError, TypeError):
             # Code page number in locale is not valid
             pass
-    except ImportError:
+    except:
         pass
 
 
 #==============================================================================
 # Settings for our MacOs X app
 #==============================================================================
+# FIXME: If/when we create new apps we need to revisit this!
 if sys.platform == 'darwin':
     from spyder.config.base import MAC_APP_NAME
     if MAC_APP_NAME in __file__:
@@ -199,12 +166,57 @@ if sys.platform == 'darwin':
             site.USER_SITE = osx_app_site.getusersitepackages()
 
 
+#==============================================================================
+# Cython support
+#==============================================================================
+RUN_CYTHON = os.environ.get("SPY_RUN_CYTHON") == "True"
+HAS_CYTHON = False
+
+if RUN_CYTHON:
+    try:
+        __import__('Cython')
+        HAS_CYTHON = True
+    except Exception:
+        pass
+
+    if HAS_CYTHON:
+        # Import pyximport to enable Cython files support for
+        # import statement
+        import pyximport
+        pyx_setup_args = {}
+
+        # Add Numpy include dir to pyximport/distutils
+        try:
+            import numpy
+            pyx_setup_args['include_dirs'] = numpy.get_include()
+        except Exception:
+            pass
+
+        # Setup pyximport and enable Cython files reload
+        pyximport.install(setup_args=pyx_setup_args, reload_support=True)
+
+
+#==============================================================================
+# Prevent subprocess.Popen calls to create visible console windows on Windows.
+# See issue #4932
+#==============================================================================
+if os.name == 'nt':
+    import subprocess
+    creation_flag = 0x08000000  # CREATE_NO_WINDOW
+
+    class SubprocessPopen(subprocess.Popen):
+        def __init__(self, *args, **kwargs):
+            kwargs['creationflags'] = creation_flag
+            super(SubprocessPopen, self).__init__(*args, **kwargs)
+
+    subprocess.Popen = SubprocessPopen
+
 #==============================================================================
 # Importing user's sitecustomize
 #==============================================================================
 try:
     import sitecustomize  #analysis:ignore
-except ImportError:
+except:
     pass
 
 
@@ -232,259 +244,44 @@ if os.environ["QT_API"] == 'pyqt':
         pass
 
 
-#==============================================================================
-# Importing matplotlib before creating the monitor.
-# This prevents a kernel crash with the inline backend in our IPython
-# consoles on Linux and Python 3 (Fixes Issue 2257)
-#==============================================================================
-try:
-    import matplotlib
-except (ImportError, UnicodeDecodeError):
-    matplotlib = None   # analysis:ignore
-
-
-#==============================================================================
-# Monitor-based functionality
-#==============================================================================
-if IS_IPYKERNEL or os.environ.get('SPYDER_SHELL_ID') is None:
-    monitor = None
-else:
-    from spyder.widgets.externalshell.monitor import Monitor
-    monitor = Monitor("127.0.0.1",
-                      int(os.environ['SPYDER_I_PORT']),
-                      int(os.environ['SPYDER_N_PORT']),
-                      os.environ['SPYDER_SHELL_ID'],
-                      float(os.environ['SPYDER_AR_TIMEOUT']),
-                      os.environ["SPYDER_AR_STATE"].lower() == "true")
-    monitor.start()
-
-    def open_in_spyder(source, lineno=1):
-        """
-        Open a source file in Spyder's editor (it could be a filename or a
-        Python module/package).
-
-        If you want to use IPython's %edit use %ed instead
-        """
-        try:
-            source = sys.modules[source]
-        except KeyError:
-            source = source
-        if not isinstance(source, basestring):
-            try:
-                source = source.__file__
-            except AttributeError:
-                raise ValueError("source argument must be either "
-                                 "a string or a module object")
-        if source.endswith('.pyc'):
-            source = source[:-1]
-        source = osp.abspath(source)
-        if osp.exists(source):
-            monitor.notify_open_file(source, lineno=lineno)
-        else:
-            _print("Can't open file %s" % source, file=sys.stderr)
-    builtins.open_in_spyder = open_in_spyder
-
-    # Our own input hook, monitor based and for Windows only
-    if os.name == 'nt' and matplotlib and not IS_IPYKERNEL:
-        # Qt imports
-        if os.environ["QT_API"] == 'pyqt5':
-            from PyQt5 import QtCore
-            from PyQt5 import QtWidgets
-        elif os.environ["QT_API"] == 'pyqt':
-            from PyQt4 import QtCore           # analysis:ignore
-            from PyQt4 import QtGui as QtWidgets
-        elif os.environ["QT_API"] == 'pyside':
-            from PySide import QtCore          # analysis:ignore
-            from PySide import QtGui as QtWidgets
-
-        def qt_nt_inputhook():
-            """Qt input hook for Windows
-
-            This input hook wait for available stdin data (notified by
-            ExternalPythonShell through the monitor's inputhook_flag
-            attribute), and in the meantime it processes Qt events.
-            """
-            # Refreshing variable explorer, except on first input hook call:
-            # (otherwise, on slow machines, this may freeze Spyder)
-            monitor.refresh_from_inputhook()
-
-            # NOTE: This is making the inputhoook to fail completely!!
-            #       That's why it's commented.
-            #try:
-                # This call fails for Python without readline support
-                # (or on Windows platforms) when PyOS_InputHook is called
-                # for the second consecutive time, because the 100-bytes
-                # stdin buffer is full.
-                # For more details, see the `PyOS_StdioReadline` function
-                # in Python source code (Parser/myreadline.c)
-            #    sys.stdin.tell()
-            #except IOError:
-            #    return 0
-
-            # Input hook
-            app = QtCore.QCoreApplication.instance()
-            if app is None:
-                app = QtWidgets.QApplication([" "])
-            if app and app.thread() is QtCore.QThread.currentThread():
-                try:
-                    timer = QtCore.QTimer()
-                    timer.timeout.connect(app.quit)
-                    monitor.toggle_inputhook_flag(False)
-                    while not monitor.inputhook_flag:
-                        timer.start(50)
-                        QtCore.QCoreApplication.exec_()
-                        timer.stop()
-                except KeyboardInterrupt:
-                    _print("\nKeyboardInterrupt - Press Enter for new prompt")
-
-                # Socket-based alternative:
-                #socket = QtNetwork.QLocalSocket()
-                #socket.connectToServer(os.environ['SPYDER_SHELL_ID'])
-                #socket.waitForConnected(-1)
-                #while not socket.waitForReadyRead(10):
-                #    timer.start(50)
-                #    QtCore.QCoreApplication.exec_()
-                #    timer.stop()
-                #socket.read(3)
-                #socket.disconnectFromServer()
-            return 0
-
-
-#==============================================================================
-# Matplotlib settings
-#==============================================================================
-if matplotlib is not None:
-    if not IS_IPYKERNEL:
-        mpl_backend = os.environ.get("SPY_MPL_BACKEND", "")
-        mpl_ion = os.environ.get("MATPLOTLIB_ION", "")
-
-        # Setting no backend if the user asks for it
-        if not mpl_backend or mpl_backend.lower() == 'none':
-            mpl_backend = ""
-
-        # Set backend automatically
-        if mpl_backend.lower() == 'automatic':
-            if not IS_EXT_INTERPRETER:
-                if os.environ["QT_API"] == 'pyqt5':
-                    mpl_backend = 'Qt5Agg'
-                else:
-                    mpl_backend = 'Qt4Agg'
-            else:
-                # Test for backend libraries on external interpreters
-                def set_mpl_backend(backend):
-                    mod, bend, qt_api = backend
-                    try:
-                        if mod:
-                            __import__(mod)
-                        if qt_api and (os.environ["QT_API"] != qt_api):
-                            return None
-                        else:
-                            matplotlib.use(bend)
-                            return bend
-                    except (ImportError, ValueError):
-                        return None
-
-                backends = [('PyQt5', 'Qt5Agg', 'pyqt5'),
-                            ('PyQt4', 'Qt4Agg', 'pyqt'),
-                            ('PySide', 'Qt4Agg', 'pyqt')]
-                if not os.name == 'nt':
-                     backends.append( ('_tkinter', 'TkAgg', None) )
-
-                for b in backends:
-                    mpl_backend = set_mpl_backend(b)
-                    if mpl_backend:
-                        break
-
-                if not mpl_backend:
-                    _print("NOTE: No suitable Matplotlib backend was found!\n"
-                           "      You won't be able to create plots\n")
-
-        # To have mpl docstrings as rst
-        matplotlib.rcParams['docstring.hardcopy'] = True
-
-        # Activate interactive mode when needed
-        if mpl_ion.lower() == "true":
-            matplotlib.rcParams['interactive'] = True
-
-        from spyder.utils import inputhooks
-        if mpl_backend:
-            import ctypes
-
-            # Grab QT_API
-            qt_api = os.environ["QT_API"]
-
-            # Setting the user defined backend
-            if not IS_EXT_INTERPRETER:
-                matplotlib.use(mpl_backend)
-
-            # Setting the right input hook according to mpl_backend,
-            # IMPORTANT NOTE: Don't try to abstract the steps to set a PyOS
-            # input hook callback in a function. It will **crash** the
-            # interpreter!!
-            if (mpl_backend == "Qt4Agg" or mpl_backend == "Qt5Agg") and \
-              os.name == 'nt' and monitor is not None:
-                # Removing PyQt4 input hook which is not working well on
-                # Windows since opening a subprocess does not attach a real
-                # console to it (with keyboard events...)
-                if qt_api == 'pyqt' or qt_api == 'pyqt5':
-                    inputhooks.remove_pyqt_inputhook()
-                # Using our own input hook
-                # NOTE: it's not working correctly for some configurations
-                # (See issue 1831)
-                callback = inputhooks.set_pyft_callback(qt_nt_inputhook)
-                pyos_ih = inputhooks.get_pyos_inputhook()
-                pyos_ih.value = ctypes.cast(callback, ctypes.c_void_p).value
-            elif mpl_backend == "Qt4Agg" and qt_api == 'pyside':
-                # PySide doesn't have an input hook, so we need to install one
-                # to be able to show plots
-                # Note: This only works well for Posix systems
-                callback = inputhooks.set_pyft_callback(inputhooks.qt4)
-                pyos_ih = inputhooks.get_pyos_inputhook()
-                pyos_ih.value = ctypes.cast(callback, ctypes.c_void_p).value
-            elif (mpl_backend != "Qt4Agg" and qt_api == 'pyqt') \
-              or (mpl_backend != "Qt5Agg" and qt_api == 'pyqt5'):
-                # Matplotlib backends install their own input hooks, so we
-                # need to remove the PyQt one to make them work
-                inputhooks.remove_pyqt_inputhook()
-        else:
-            inputhooks.remove_pyqt_inputhook()
-    else:
-        # To have mpl docstrings as rst
-        matplotlib.rcParams['docstring.hardcopy'] = True
-
-
 #==============================================================================
 # IPython kernel adjustments
 #==============================================================================
-if IS_IPYKERNEL:
-    # Use ipydb as the debugger to patch on IPython consoles
-    from IPython.core.debugger import Pdb as ipyPdb
-    pdb.Pdb = ipyPdb
+# Use ipydb as the debugger to patch on IPython consoles
+from IPython.core.debugger import Pdb as ipyPdb
+pdb.Pdb = ipyPdb
 
-    # Patch unittest.main so that errors are printed directly in the console.
-    # See http://comments.gmane.org/gmane.comp.python.ipython.devel/10557
-    # Fixes Issue 1370
-    import unittest
-    from unittest import TestProgram
-    class IPyTesProgram(TestProgram):
-        def __init__(self, *args, **kwargs):
-            test_runner = unittest.TextTestRunner(stream=sys.stderr)
-            kwargs['testRunner'] = kwargs.pop('testRunner', test_runner)
-            kwargs['exit'] = False
-            TestProgram.__init__(self, *args, **kwargs)
-    unittest.main = IPyTesProgram
+# Patch unittest.main so that errors are printed directly in the console.
+# See http://comments.gmane.org/gmane.comp.python.ipython.devel/10557
+# Fixes Issue 1370
+import unittest
+from unittest import TestProgram
+class IPyTesProgram(TestProgram):
+    def __init__(self, *args, **kwargs):
+        test_runner = unittest.TextTestRunner(stream=sys.stderr)
+        kwargs['testRunner'] = kwargs.pop('testRunner', test_runner)
+        kwargs['exit'] = False
+        TestProgram.__init__(self, *args, **kwargs)
+unittest.main = IPyTesProgram
+
+# Patch ipykernel to avoid errors when setting the Qt5 Matplotlib
+# backemd
+# Fixes Issue 6091
+import ipykernel
+import IPython
+if LooseVersion(ipykernel.__version__) <= LooseVersion('4.7.0'):
+    if ((PY2 and LooseVersion(IPython.__version__) >= LooseVersion('5.5.0')) or
+        (not PY2 and LooseVersion(IPython.__version__) >= LooseVersion('6.2.0'))
+       ):
+        from ipykernel import eventloops
+        eventloops.loop_map['qt'] = eventloops.loop_map['qt5']
 
 
 #==============================================================================
 # Pandas adjustments
 #==============================================================================
 try:
-    # Make Pandas recognize our Jupyter consoles as proper qtconsoles
-    # Fixes Issue 2015
-    def in_qtconsole():
-        return True
     import pandas as pd
-    pd.core.common.in_qtconsole = in_qtconsole
 
     # Set Pandas output encoding
     pd.options.display.encoding = 'utf-8'
@@ -494,7 +291,6 @@ try:
     # >>> import pandas as pd, numpy as np
     # >>> pd.Series([np.nan,np.nan,np.nan],index=[1,2,3])
     # Fixes Issue 2991
-    import warnings
     # For 0.18-
     warnings.filterwarnings(action='ignore', category=RuntimeWarning,
                             module='pandas.core.format',
@@ -503,7 +299,7 @@ try:
     warnings.filterwarnings(action='ignore', category=RuntimeWarning,
                             module='pandas.formats.format',
                             message=".*invalid value encountered in.*")
-except (ImportError, AttributeError):
+except:
     pass
 
 
@@ -511,7 +307,9 @@ except (ImportError, AttributeError):
 # Pdb adjustments
 #==============================================================================
 class SpyderPdb(pdb.Pdb):
+
     send_initial_notification = True
+    starting = True
 
     def set_spyder_breakpoints(self):
         self.clear_all_breaks()
@@ -537,6 +335,11 @@ class SpyderPdb(pdb.Pdb):
     def notify_spyder(self, frame):
         if not frame:
             return
+
+        from IPython.core.getipython import get_ipython
+        kernel = get_ipython().kernel
+
+        # Get filename and line number of the current frame
         fname = self.canonic(frame.f_code.co_filename)
         if PY2:
             try:
@@ -544,20 +347,36 @@ class SpyderPdb(pdb.Pdb):
             except TypeError:
                 pass
         lineno = frame.f_lineno
+
+        # Jump to first breakpoint.
+        # Fixes issue 2034
+        if self.starting:
+            # Only run this after a Pdb session is created
+            self.starting = False
+
+            # Get all breakpoints for the file we're going to debug
+            breaks = self.get_file_breaks(frame.f_code.co_filename)
+
+            # Do 'continue' if the first breakpoint is *not* placed
+            # where the debugger is going to land.
+            # Fixes issue 4681
+            if breaks and lineno != breaks[0] and osp.isfile(fname):
+                kernel.pdb_continue()
+
+        # Set step of the current frame (if any)
+        step = {}
         if isinstance(fname, basestring) and isinstance(lineno, int):
             if osp.isfile(fname):
-                if IS_IPYKERNEL:
-                    from IPython.core.getipython import get_ipython
-                    ipython_shell = get_ipython()
-                    if ipython_shell:
-                        step = dict(fname=fname, lineno=lineno)
-                        ipython_shell.kernel._pdb_step = step
-                elif monitor is not None:
-                    monitor.notify_pdb_step(fname, lineno)
-                    time.sleep(0.1)
+                step = dict(fname=fname, lineno=lineno)
+
+        # Publish Pdb state so we can update the Variable Explorer
+        # and the Editor on the Spyder side
+        kernel._pdb_step = step
+        kernel.publish_pdb_state()
 
 pdb.Pdb = SpyderPdb
 
+
 #XXX: I know, this function is now also implemented as is in utils/misc.py but
 #     I'm kind of reluctant to import spyder in sitecustomize, even if this
 #     import is very clean.
@@ -580,7 +399,7 @@ def monkeypatch_method(cls, patch_name):
         if old_func is not None:
             # Add the old func to a list of old funcs.
             old_ref = "_old_%s_%s" % (patch_name, fname)
-            #print(old_ref, old_func)
+
             old_attr = getattr(cls, old_ref, None)
             if old_attr is None:
                 setattr(cls, old_ref, old_func)
@@ -591,6 +410,13 @@ def monkeypatch_method(cls, patch_name):
         return func
     return decorator
 
+
+@monkeypatch_method(pdb.Pdb, 'Pdb')
+def __init__(self, completekey='tab', stdin=None, stdout=None,
+             skip=None, nosigint=False):
+    self._old_Pdb___init__()
+
+
 @monkeypatch_method(pdb.Pdb, 'Pdb')
 def user_return(self, frame, return_value):
     """This function is called when a return trap is set here."""
@@ -603,27 +429,46 @@ def user_return(self, frame, return_value):
         self._wait_for_mainpyfile = 0
     self._old_Pdb_user_return(frame, return_value)
 
+
 @monkeypatch_method(pdb.Pdb, 'Pdb')
 def interaction(self, frame, traceback):
-    self.setup(frame, traceback)
-    if self.send_initial_notification:
-        self.notify_spyder(frame) #-----Spyder-specific-----------------------
-    self.print_stack_entry(self.stack[self.curindex])
-    self.cmdloop()
-    self.forget()
+    if frame is not None and "sitecustomize.py" in frame.f_code.co_filename:
+        self.run('exit')
+    else:
+        self.setup(frame, traceback)
+        if self.send_initial_notification:
+            self.notify_spyder(frame)
+        self.print_stack_entry(self.stack[self.curindex])
+        self._cmdloop()
+        self.forget()
+
+
+@monkeypatch_method(pdb.Pdb, 'Pdb')
+def _cmdloop(self):
+    while True:
+        try:
+            # keyboard interrupts allow for an easy way to cancel
+            # the current command, so allow them during interactive input
+            self.allow_kbdint = True
+            self.cmdloop()
+            self.allow_kbdint = False
+            break
+        except KeyboardInterrupt:
+            _print("--KeyboardInterrupt--\n"
+                   "For copying text while debugging, use Ctrl+Shift+C",
+                   file=self.stdout)
+
 
 @monkeypatch_method(pdb.Pdb, 'Pdb')
 def reset(self):
     self._old_Pdb_reset()
-    if IS_IPYKERNEL:
-        from IPython.core.getipython import get_ipython
-        ipython_shell = get_ipython()
-        if ipython_shell:
-            ipython_shell.kernel._register_pdb_session(self)
-    elif monitor is not None:
-        monitor.register_pdb_session(self)
+
+    from IPython.core.getipython import get_ipython
+    kernel = get_ipython().kernel
+    kernel._register_pdb_session(self)
     self.set_spyder_breakpoints()
 
+
 #XXX: notify spyder on any pdb command (is that good or too lazy? i.e. is more
 #     specific behaviour desired?)
 @monkeypatch_method(pdb.Pdb, 'Pdb')
@@ -631,6 +476,7 @@ def postcmd(self, stop, line):
     self.notify_spyder(self.curframe)
     return self._old_Pdb_postcmd(stop, line)
 
+
 # Breakpoints don't work for files with non-ascii chars in Python 2
 # Fixes Issue 1484
 if PY2:
@@ -671,8 +517,7 @@ if PY2:
 # 'site-packages' directory from sys.path!
 #==============================================================================
 try:
-    sys.path.remove(osp.join(spyder_path, "spyder", "widgets",
-                             "externalshell"))
+    sys.path.remove(osp.join(spyder_path, "spyder", "utils", "site"))
 except ValueError:
     pass
 
@@ -701,6 +546,9 @@ class UserModuleReloader(object):
         self.previous_modules = list(sys.modules.keys())
 
     def is_module_blacklisted(self, modname, modpath):
+        if HAS_CYTHON:
+            # Don't return cached inline compiled .PYX files
+            return True
         for path in [sys.prefix]+self.pathlist:
             if modpath.startswith(path):
                 return True
@@ -741,13 +589,9 @@ def clear_post_mortem():
     """
     Remove the post mortem excepthook and replace with a standard one.
     """
-    if IS_IPYKERNEL:
-        from IPython.core.getipython import get_ipython
-        ipython_shell = get_ipython()
-        if ipython_shell:
-            ipython_shell.set_custom_exc((), None)
-    else:
-        sys.excepthook = sys.__excepthook__
+    from IPython.core.getipython import get_ipython
+    ipython_shell = get_ipython()
+    ipython_shell.set_custom_exc((), None)
 
 
 def post_mortem_excepthook(type, value, tb):
@@ -756,14 +600,11 @@ def post_mortem_excepthook(type, value, tb):
     mortem debugging.
     """
     clear_post_mortem()
-    if IS_IPYKERNEL:
-        from IPython.core.getipython import get_ipython
-        ipython_shell = get_ipython()
-        ipython_shell.showtraceback((type, value, tb))
-        p = pdb.Pdb(ipython_shell.colors)
-    else:
-        traceback.print_exception(type, value, tb, file=sys.stderr)
-        p = pdb.Pdb()
+
+    from IPython.core.getipython import get_ipython
+    ipython_shell = get_ipython()
+    ipython_shell.showtraceback((type, value, tb))
+    p = pdb.Pdb(ipython_shell.colors)
 
     if not type == SyntaxError:
         # wait for stderr to print (stderr.flush does not work in this case)
@@ -789,15 +630,12 @@ def set_post_mortem():
     """
     Enable the post mortem debugging excepthook.
     """
-    if IS_IPYKERNEL:
-        from IPython.core.getipython import get_ipython
-        def ipython_post_mortem_debug(shell, etype, evalue, tb,
-                   tb_offset=None):
-            post_mortem_excepthook(etype, evalue, tb)
-        ipython_shell = get_ipython()
-        ipython_shell.set_custom_exc((Exception,), ipython_post_mortem_debug)
-    else:
-        sys.excepthook = post_mortem_excepthook
+    from IPython.core.getipython import get_ipython
+    def ipython_post_mortem_debug(shell, etype, evalue, tb,
+               tb_offset=None):
+        post_mortem_excepthook(etype, evalue, tb)
+    ipython_shell = get_ipython()
+    ipython_shell.set_custom_exc((Exception,), ipython_post_mortem_debug)
 
 # Add post mortem debugging if requested and in a dedicated interpreter
 # existing interpreters use "runfile" below
@@ -809,16 +647,10 @@ if "SPYDER_EXCEPTHOOK" in os.environ:
 # runfile and debugfile commands
 #==============================================================================
 def _get_globals():
-    """Return current Python interpreter globals namespace"""
-    from __main__ import __dict__ as namespace
-    shell = namespace.get('__ipythonshell__')
-    if shell is not None and hasattr(shell, 'user_ns'):
-        # IPython 0.13+ kernel
-        return shell.user_ns
-    else:
-        # Python interpreter
-        return namespace
-    return namespace
+    """Return current namespace"""
+    from IPython.core.getipython import get_ipython
+    ipython_shell = get_ipython()
+    return ipython_shell.user_ns
 
 
 def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
@@ -835,14 +667,14 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
         # AttributeError --> systematically raised in Python 3
         pass
     global __umr__
-    if os.environ.get("UMR_ENABLED", "").lower() == "true":
+    if os.environ.get("SPY_UMR_ENABLED", "").lower() == "true":
         if __umr__ is None:
-            namelist = os.environ.get("UMR_NAMELIST", None)
+            namelist = os.environ.get("SPY_UMR_NAMELIST", None)
             if namelist is not None:
                 namelist = namelist.split(',')
             __umr__ = UserModuleReloader(namelist=namelist)
         else:
-            verbose = os.environ.get("UMR_VERBOSE", "").lower() == "true"
+            verbose = os.environ.get("SPY_UMR_VERBOSE", "").lower() == "true"
             __umr__.run(verbose=verbose)
     if args is not None and not isinstance(args, basestring):
         raise TypeError("expected a character buffer object")
@@ -863,7 +695,15 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
         os.chdir(wdir)
     if post_mortem:
         set_post_mortem()
-    execfile(filename, namespace)
+    if HAS_CYTHON:
+        # Cython files
+        with io.open(filename, encoding='utf-8') as f:
+            from IPython.core.getipython import get_ipython
+            ipython_shell = get_ipython()
+            ipython_shell.run_cell_magic('cython', '', f.read())
+    else:
+        execfile(filename, namespace)
+
     clear_post_mortem()
     sys.argv = ['']
     namespace.pop('__file__')
@@ -890,52 +730,6 @@ def debugfile(filename, args=None, wdir=None, post_mortem=False):
 builtins.debugfile = debugfile
 
 
-#==============================================================================
-# Evaluate external commands
-#==============================================================================
-def evalsc(command):
-    """Evaluate special commands
-    (analog to IPython's magic commands but far less powerful/complete)"""
-    assert command.startswith('%')
-    from spyder.utils import programs
-
-    namespace = _get_globals()
-    command = command[1:].strip()  # Remove leading %
-
-    import re
-    clear_match = re.match(r"^clear ([a-zA-Z0-9_, ]+)", command)
-    cd_match = re.match(r"^cd \"?\'?([a-zA-Z0-9_\ \:\\\/\.]+)", command)
-
-    if cd_match:
-        os.chdir(eval('r"%s"' % cd_match.groups()[0].strip()))
-    elif clear_match:
-        varnames = clear_match.groups()[0].replace(' ', '').split(',')
-        for varname in varnames:
-            try:
-                namespace.pop(varname)
-            except KeyError:
-                pass
-    elif command in ('cd', 'pwd'):
-        try:
-            _print(os.getcwdu())
-        except AttributeError:
-            _print(os.getcwd())
-    elif command == 'ls':
-        if os.name == 'nt':
-            programs.run_shell_command('dir')
-            _print('\n')
-        else:
-            programs.run_shell_command('ls')
-            _print('\n')
-    elif command == 'scientific':
-        from spyder.config import base
-        execfile(base.SCIENTIFIC_STARTUP, namespace)
-    else:
-        raise NotImplementedError("Unsupported command: '%s'" % command)
-
-builtins.evalsc = evalsc
-
-
 #==============================================================================
 # Restoring original PYTHONPATH
 #==============================================================================
