@@ -73,7 +73,9 @@ class DebuggingWidget(RichJupyterWidget):
 
         def callback(line):
             # Save history to browse it later
-            self._control.history.append(line)
+            if not (len(self._control.history) > 0
+                    and self._control.history[-1] == line):
+                self._control.history.append(line)
 
             # This is the Spyder addition: add a %plot magic to display
             # plots while debugging
@@ -99,9 +101,10 @@ class DebuggingWidget(RichJupyterWidget):
             elif key == Qt.Key_Down:
                 self._control.browse_history(backward=False)
                 return True
-            else:
-                return super(DebuggingWidget,
-                             self)._event_filter_console_keypress(event)
+            elif key in (Qt.Key_Return, Qt.Key_Enter):
+                self._control.reset_search_pos()
+            return super(DebuggingWidget,
+                         self)._event_filter_console_keypress(event)
         else:
             return super(DebuggingWidget,
                          self)._event_filter_console_keypress(event)
